/* Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

--- Rename `reset` column to `type` to make it more appropriate
alter table token
rename column reset to "type";

alter table token
alter column "type" type text
using (case when "type" then 'reset' else '' end);


--- Add "status" field to envc table to identify current status of license deployment
--- Add "depl_info" column that will store any information required/generated during license deployment
alter table envc
add column depl_status text default 'finished',
add column depl_info json default '{}';


--- Increase size of "type" column in cluster_shared_data table
--- Add "file_data" column to cluster_shared_data table to support file sharing over cluster
--- Add a reference
ALTER TABLE cluster_shared_data ALTER COLUMN type TYPE character varying(1024);
ALTER TABLE cluster_shared_data ADD COLUMN file_data bytea;
ALTER TABLE cluster_shared_data ADD CONSTRAINT unique_type UNIQUE (type);


--- Add "shared_data_type" column to cluster_command table that points to "type" column of cluster_shared_data table
ALTER TABLE cluster_command ADD COLUMN shared_data_type character varying(1024);
ALTER TABLE cluster_command
   ADD CONSTRAINT fk_shared_data_type
   FOREIGN KEY (shared_data_type)
   REFERENCES cluster_shared_data(type);


--- Add "workflow_threshold" config parameter
INSERT INTO configprops ("section", "key_value", "value_value", "public", data_type) VALUES ('HA', 'workflow_threshold', '100', True, 'int');



--- Add "restricted_token_lifetime" config parameter that identifies the lifetime of restricted token(in minutes) used in
--- manual input resume/approve
INSERT INTO configprops ("section", "key_value", "value_value", "public", data_type) VALUES ('TOKEN', 'restricted_token_lifetime', '2880', True, 'int');

--- Introduced a new data_type "encrypted_json", this will remove the overhead of encrypting/decrypting data everytime when using
--- proxy_details are saved in encrypted format but with type "str", convert the type to "encrypted_json"
UPDATE configprops SET data_type='encrypted_json'  WHERE section = 'PROXY' AND key_value = 'proxy_settings' ;

