/* Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */
-- NFA Migration

ALTER TABLE nfa DROP COLUMN if exists service_mask;
ALTER TABLE nfa ADD COLUMN if not exists title character varying(32);
ALTER TABLE nfa ADD COLUMN if not exists config_schema jsonb;
ALTER TABLE nfa ADD COLUMN if not exists config text;
ALTER TABLE nfa ADD COLUMN if not exists traits character varying(32)[];
ALTER TABLE nfa ADD COLUMN if not exists timeout integer;
ALTER TABLE nfa ADD COLUMN if not exists configured bool DEFAULT FALSE;
ALTER TABLE nfa ADD COLUMN if not exists system bool DEFAULT FALSE;

-- update auth
UPDATE nfa set
    title='auth',
    service_name='auth',
    available=True,
    config_schema='{}',
    config='',
    traits = '{}',
    system = True,
    timeout=0
 WHERE nfa_id=7;

-- update telesign
UPDATE nfa set
    title='Telesign',
    service_name='Telesign',
    available=false,
    config_schema='{"global":{"inputFields":[{"title":"Customer ID","name":"customer_id","required":"True","visible":"True","editable":"True","type":"text","tooltip":"Customer ID for your Telesign account","description":"","placeholder":""},{"title":"API KEY","name":"api_key","required":"True","visible":"True","editable":"True","type":"password","tooltip":"API Key for your Telesign account","description":"","placeholder":""},{"title":"Minimum OTP Value for SMS","name":"sms_begin_range","required":"True","visible":"True","editable":"True","type":"decimal","tooltip":"Smallest number that can be used as an OTP while sending a text message ","description":"","placeholder":""},{"title":"Maximum OTP Value for SMS","name":"sms_end_range","required":"True","visible":"True","editable":"True","type":"decimal","tooltip":"Largest number that can be used as an OTP while sending a text message ","description":"","placeholder":""},{"title":"Minimum OTP Value for voice","name":"voice_begin_range","required":"True","visible":"True","editable":"True","type":"decimal","tooltip":"Smallest number that can be used as an OTP while sending a voice message ","description":"","placeholder":""},{"title":"Maximum OTP Value for voice","name":"voice_end_range","required":"True","visible":"True","editable":"True","type":"decimal","tooltip":"Largest number that can be used as an OTP while sending a voice message ","description":"","placeholder":""}],"outputFields":[],"systemFields":[]},"user":{"inputFields":[{"title":"Phone number","name":"phone","required":"True","visible":"True","editable":"True","type":"phone","tooltip":"Phone number to be configured to receive OTP messages","description":"","placeholder":""},{"title":"Mode","name":"mode","required":"True","visible":"True","editable":"True","type":"select","options":["SMS","Voice"],"tooltip":"Mode to be used to deliver the OTP","description":"","placeholder":""}],"outputFields":[],"systemFields":[]}}',
    config='',
    traits = '{trait_configure_self,trait_custom_message,trait_generate_code,trait_health_check}',
    system = True,
    timeout=60
 WHERE nfa_id=8;

-- update google auth
UPDATE nfa set
    title='Google Authenticator',
    service_name='GoogleAuthenticator',
    available=false,
    config_schema='{"user":{"inputFields":[{"name":"email","type":"email","title":"Email","tooltip":"","editable":"True","visible":"True","required":"True","description":"","placeholder":""}],"outputFields":[{"name":"qr","type":"img","title":"You need Google Authenticator to complete this process.<br/>Scan the QR code in your app","tooltip":"","editable":"False","visible":"True","required":"True","description":"QR Code to configure Google Authenticator","placeholder":""}],"systemFields":[{"name":"secret","type":"text","title":"Secret","tooltip":"","editable":"False","visible":"True","required":"True","description":"","placeholder":"Secret"}]},"global":{"inputFields":[{"name":"issuer","type":"text","title":"Issuer name","tooltip":"The issuer parameter will be used as the name of the provider or service this account is associated with","editable":"True","visible":"True","required":"True","description":"","placeholder":""}],"outputFields":[],"systemFields":[]}}',
    config='',
    traits = '{trait_configure_self,trait_custom_message,trait_verify_code}',
    system = True,
    timeout=0
 WHERE nfa_id=9;

-- Add Constraints
ALTER TABLE nfa ALTER COLUMN title SET NOT NULL;
ALTER TABLE nfa ALTER COLUMN config_schema SET NOT NULL;
-- End NFA Migration

-- NFAUser Migration

ALTER TABLE nfauser DROP COLUMN if exists  processed;
ALTER TABLE nfauser ADD COLUMN if not exists config text;
ALTER TABLE nfauser ADD COLUMN if not exists enabled bool DEFAULT FALSE;

-- migrate old telesign global config
-- ignore-failure
update nfa set config = concat('{"api_key":"',(select value_value from configprops where key_value='api_key'),'"',',','"customer_id":"',(select value_value from configprops where key_value='customer_id'),'"',',"sms_begin_range": "100000","sms_end_range": "999999","voice_begin_range": "10000","voice_end_range": "99999"}') where exists (select value_value from configprops where key_value = 'customer_id' and value_value is not null) and exists (select value_value from configprops where key_value = 'api_key' and value_value is not null) and nfa_id=8;

-- activate telesign
-- ignore-failure
update nfa set available = true, configured = true where exists (select value_value from configprops where key_value = 'customer_id' and value_value is not null) and exists (select value_value from configprops where key_value = 'api_key' and value_value is not null) and nfa_id=8;

-- migrate old telesign config to nfauser
-- ignore-failure
update nfauser set config = concat('{"phone":"+',(select phone from nfausercfg where nfauser.uuid = nfausercfg.uuid and nfauser.nfa_id = nfausercfg.nfa_id),'","mode":"',lower((select method from nfausercfg where nfauser.uuid = nfausercfg.uuid and nfauser.nfa_id = nfausercfg.nfa_id)),'"}'), enabled=True where nfa_id=8 and exists (select * from nfausercfg where nfausercfg.uuid= nfauser.uuid) and exists (select nfa_id from nfa where nfa_id=8 and available=true);




-- Delete telesign entries from configprops
delete from configprops where section='TELESIGN';
drop table nfausercfg;

-- clear off duplicates if any
delete from nfauser n1 where (select count(uuid) from nfauser n2 where n1.nfauser_id>n2.nfauser_id and n2.uuid=n1.uuid and n2.nfa_id=n1.nfa_id)>0;

-- add constraints to nfauser
ALTER TABLE ONLY nfauser
    ADD CONSTRAINT nfauser_uuid_nfa_id UNIQUE (uuid,nfa_id);

ALTER TABLE nfauser ALTER COLUMN uuid SET NOT NULL;
ALTER TABLE nfauser ALTER COLUMN nfa_id SET NOT NULL;
ALTER TABLE nfauser ALTER COLUMN precedence SET NOT NULL;

-- END NFAUser Migration

-- NFAResponse Migration
truncate nfaresponse;
ALTER TABLE nfaresponse ADD COLUMN if not exists jti character varying(64) NOT NULL;
-- END NFAResponse Migration

-- NFA Status Migration
CREATE SEQUENCE if not exists nfa_status_id_seq
    START WITH 3
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

CREATE TABLE if not exists nfa_status (
    nfa_status_id integer DEFAULT nextval('nfa_status_id_seq'::regclass) NOT NULL NOT NULL,
    status integer,
    last_success timestamp(6) without time zone,
    last_attempt timestamp(6) without time zone,
    grace_until timestamp(6) without time zone,
    uuid character varying(64)
);

ALTER TABLE nfa_status_id_seq OWNER TO cyberpgsql;
ALTER SEQUENCE nfa_status_id_seq OWNED BY nfa_status.nfa_status_id;

ALTER TABLE ONLY nfa_status
    ADD CONSTRAINT nfa_status_uuid_key UNIQUE (uuid);

-- insert users
insert into nfa_status(status,uuid) select 2,uuid from userinfo where user_type=2;

-- update nfa_status for those users whose nfa is configured
update nfa_status set status=1 where exists (select * from nfauser where nfauser.uuid= nfa_status.uuid and nfauser.enabled=True);


-- END NFA Status Migration

-- NFA Auth Status Migration

CREATE SEQUENCE if not exists nfa_auth_status_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


CREATE TABLE if not exists nfa_auth_status (
    nfa_auth_status_id integer DEFAULT nextval('nfa_auth_status_id_seq'::regclass) NOT NULL,
    uuid character varying(64) NOT NULL,
    jti character varying(64) NOT NULL,
    executed_nfa_id Integer NOT NULL,
    processed boolean,
    expiration timestamp without time zone
);

ALTER TABLE nfa_auth_status_id_seq OWNER TO cyberpgsql;
ALTER SEQUENCE nfa_auth_status_id_seq OWNED BY nfa_auth_status.nfa_auth_status_id;


ALTER TABLE ONLY nfa_auth_status
    ADD CONSTRAINT nfa_auth_status_nfa_id_fk FOREIGN KEY (executed_nfa_id) REFERENCES nfa(nfa_id) ON DELETE CASCADE;

ALTER TABLE ONLY nfauser
    ADD CONSTRAINT nfauser_nfa_id_idx FOREIGN KEY (nfa_id) REFERENCES nfa(nfa_id) ON DELETE CASCADE;