#!/opt/cyops-auth/.env/bin/python3
""" Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """

import sys, subprocess, argparse

sys.path.append('/opt/cyops-auth')

from utilities.ha.conf import read_config_file
from utilities.ha.consts import PG_PWD, PG_USER, PG_DEFAULT_DATABASE, POSTGRES_SECTION
from utilities.ha.common_utils import execute_cmd
from utilities.util import decrypt_util

def define_parser(parser):
    parser.add_argument('--command', help="Command name", dest='cmd', type=str, required=True)
    parser.add_argument('--input-args', help="Input arguments for command", dest='input', type=str, required=False)

def execute_pg_db_size_cmd():
    current_config = read_config_file()
    pg_config = current_config.get(POSTGRES_SECTION)
    pg_password = decrypt_util(pg_config.get(PG_PWD))
    cmd = f"psql -U {pg_config.get(PG_USER)} {PG_DEFAULT_DATABASE} -t -c " \
          f"\"SHOW data_directory;\" | xargs -i du -sh {{}} | awk '{{print $1}}'"
    return execute_cmd(cmd, env={'PGPASSWORD': pg_password})


def execute_update_es_credentials(es_user_pass):
    conf_util = "/opt/cyops/scripts/confUtil.py"
    db_config = "/opt/cyops/configs/database/db_config.yml"
    executable = "python3 " + conf_util + " -f " + db_config + " "
    cmd = executable + " -k es_user -v elastic"
    execute_cmd(cmd)
    cmd = executable + " -k secret -v " + es_user_pass
    execute_cmd(cmd)
    cmd = executable + " -k use_ssl -v true"
    execute_cmd(cmd)
    cmd = "systemctl restart cyops-search"
    return execute_cmd(cmd)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    define_parser(parser)
    args = parser.parse_args()

    if args.cmd == 'get-postgresql-db-size':
        exit_code, out, err = execute_pg_db_size_cmd()
        sys.stderr.write(err.strip()) if exit_code == 1 else sys.stdout.write(out.strip())
        exit(exit_code)
    elif args.cmd == 'update-es-credentials':
        exit_code, out, err = execute_update_es_credentials(args.input)
        sys.stderr.write(err.strip()) if exit_code == 1 else sys.stdout.write(out.strip())
        exit(exit_code)

    sys.stderr.write("Command [" + args.cmd + "] is not valid")
    exit(1)
